

#include <cstdlib>
#include <cstdio>
#include <math.h>
#include <string.h>
#include<iostream>
#include "nbody.h"

using namespace std;

// makes a density histogram in x,y in format suitable for splot/gnuplot

void phist(char *dfilename,  double *darr, 
            double xmin, double xmax, double dx, 
            double ymin, double ymax, double dy);
void mkhist(float *Pos,  int numBodies, double *darr, double incl,
            double xmin, double xmax, double dx, 
            double ymin, double ymax, double dy);

int main(int argc, char *argv[])
{
   if (argc<5){
      printf("mkhist.out filename.h5part rmin dx incl(deg)\n");
      exit(0);
// edge on is 90: face-on is 0
   }
   SIMinfo siminfo;
   char filename[100];
   char ofilename[100];
   strcpy(filename,argv[1]);
   info_h5part(filename, &siminfo); // get number of bodies

   float *hPos, *hVel;
   int numBodies = siminfo.numBodies;
   hPos = new float[numBodies*4];
   hVel = new float[numBodies*4];
   int timestep=0;
   read_h5part(filename, hPos, hVel,  timestep); // get data

   strcpy(ofilename,filename);
   int pch = strcspn(ofilename,".");
   ofilename[pch]='\0';
   strcat(ofilename,".d");
   printf("ofilename=%s \n",ofilename);

   double xmax,xmin,dx,ymax,ymin,dy;
   double incl;
   sscanf(argv[2],"%lf",&xmax);
   sscanf(argv[3],"%lf",&dx);
   sscanf(argv[4],"%lf",&incl);
   incl*= M_PI/180.0;
   ymax = xmax;  xmin=-xmax;  ymin = -ymax; dy = dx;
   int nx = int((xmax-xmin)/dx)+1;
   int ny = int((ymax-ymin)/dy)+1;
   printf("nx,ny %d %d\n",nx,ny); fflush(stdout);
   double *darr;
   darr = new double[nx*ny];

   // make histogram
   mkhist(hPos, numBodies, darr, incl, 
          xmin, xmax, dx, ymin, ymax, dy);
   // print it out
   phist(ofilename,  darr, 
         xmin, xmax, dx, ymin, ymax, dy);

   delete [] hPos;
   delete [] hVel;
   delete [] darr;


}

// returns a histogram density per unit area
//   inclination in radians, projecting y only
void mkhist(float *hPos,  int NumBodies, double *darr,  double incl,
            double xmin, double xmax, double dx, 
            double ymin, double ymax, double dy)
{
  double x,y,z,m;
  double area = dx*dy;
  double xproj,yproj;
  int nx = int((xmax-xmin)/dx)+1;
  int ny = int((ymax-ymin)/dy)+1;
  for(int i=0;i<nx*ny;i++) darr[i] = 0.0;
  double x0,y0,z0;
  x0 = hPos[4*0+0]; y0 = hPos[4*0+1]; z0 = hPos[4*0+2]; 
  
  for (int i=0;i<NumBodies;i++){
     x = hPos[4*i+0]; y = hPos[4*i+1]; z = hPos[4*i+2]; m = hPos[4*i+3];
     xproj = x-x0; yproj = (y-y0)*cos(incl) + (z-z0)*sin(incl);
     int xi = int((xproj - xmin)/dx);
     int yi = int((yproj - ymin)/dy);
     if ((xi>=0)&&(xi<nx)){
       if ((yi>=0)&&(yi<ny)){
         darr[xi + yi*nx] += m/area;
       }
     }

  }
}

// printf ascii density table in gnuplot form
void phist(char *dfilename,  double *darr, 
            double xmin, double xmax, double dx, 
            double ymin, double ymax, double dy) 
{
  FILE *fpo = fopen(dfilename, "w");
  int nx = int((xmax-xmin)/dx)+1;
  int ny = int((ymax-ymin)/dy)+1;
  int i,j;
  for(i=0;i<nx;i++){
    double x = xmin + i*dx;
    for(j=0;j<ny;j++){
      double y = ymin + j*dy;
      fprintf(fpo,"%.3f %.3f %.4e \n",x,y,darr[i + j*nx]);
    }
    fprintf(fpo," \n");
  }
  fclose(fpo);
}

