

//#include <paramgl.h>
#include <cstdlib>
#include <cstdio>
#include <algorithm>
#include <assert.h>
#include <math.h>
#include <cutil.h>
#include <string.h>
#include"nbody.h"

#define computeE 1 //if 1 compute energy on gpu at each timestep

#define threadsperblock 256  // hardset here

// reads in parameter file a1
// if you give it a number on the command line it does a restart at this timestep

// ouput filename manipulator
void mkofilename(char *pfile, char *ofile, int i);

int main(int argc, char *argv[])
{
   int p; 
   p = threadsperblock;
   // number of threads per block is p 
   // number of blocks is numBodies/p
   int numBodies;   // number of bodies in N-body simulation
   SIMinfo siminfo; // structure for simulation information
   char parmfile[300]; // parameter file name

   strcpy(parmfile,"a1"); // name hardset right now
   sread(parmfile,&siminfo);   // read in parameters to structure
   numBodies = siminfo.numBodies;

   // CPU data
   float* hPos; // 4 vectors, x,y,z,m
   float* hVel; // 4 vector, vx,vy,vz,unused
   // GPU data
   float* m_dPos[2]; // recast as float4 during integration
   float* m_dVel[2]; // these are 2 times x,y,z,m and 2 times vx,vy,vz,unused
   // the two arrays are alternating current and past values for positions and velocities

   checkCUDA(); // device init
   printf("num = %d\n",numBodies);
   printf("soft2= %.2e\n",siminfo.softening);
//   printf("damp= %.2f\n",siminfo.damping);
   printf("niter= %d\n",siminfo.niter);  // number of data outs
   printf("ndt= %d\n",siminfo.ndt);  // number of timesteps per dataout
   printf("iter = %.2f periods\n",
      (float)(siminfo.ndt)*siminfo.deltaTime/(2.0*M_PI));

// send softening to GPU
//   setDeviceSoftening(siminfo.softening); 
// not working last I checked

   // allocate CPU data
   hPos = new float[numBodies*4]; 
   hVel = new float[numBodies*4];
  // we allocate floats  recast as float4 later on
   memset(hPos, 0, numBodies*4*sizeof(float));
   memset(hVel, 0, numBodies*4*sizeof(float));
   // hPos includes x,y,z,m 
   // hVel includes vx,vy,vz and last vector part unused 

   int istart;
   char filename[300];
   double time;
   if (argc==1) {  // if run the first time at time =0
     istart = 0;
     // read in the initial data from paramfile.h5part
     strcpy(filename,parmfile);
     strcat(filename,".h5part");
     read_h5part(filename, hPos, hVel,  0); // get data
     printf("data read in %s\n",filename); fflush(stdout);
     time=0.0; 
     siminfo.time = 0.0;
   }
   else { // restart on file istart
      sscanf(argv[1],"%d",&istart);
      mkofilename(parmfile, filename, istart); // create filename for iter step istart
      read_h5part(filename, hPos, hVel,  0); // get data
      printf("data read in %s\n",filename); fflush(stdout);
      time=siminfo.time; 
   }

   // allocate float4 data on board GPU
   allocateNBodyArrays(m_dPos, numBodies);
   allocateNBodyArrays(m_dVel, numBodies);

   int iold=0;
   int inew=1; 
   // load initial particle positions and velocities 
   copyArrayToDevice(m_dPos[inew], hPos, numBodies); // redundant ok
   copyArrayToDevice(m_dVel[inew], hVel, numBodies);
   copyArrayToDevice(m_dPos[iold], hPos, numBodies);
   copyArrayToDevice(m_dVel[iold], hVel, numBodies);

//   printf("data copied to device\n"); fflush(stdout);

   // only store output and compute energy each iteration
   int i=0;
   for (i=istart;i<siminfo.niter;i++){  // loop iterations
       // get data from device
       copyArrayFromDevice(hPos, m_dPos[inew], numBodies);
       copyArrayFromDevice(hVel, m_dVel[inew], numBodies);
       correctcom(hPos, hVel, numBodies); // subtract center of mass and vel
       copyArrayToDevice(m_dPos[inew], hPos, numBodies); 
       copyArrayToDevice(m_dVel[inew], hVel, numBodies);
       mkofilename(parmfile, filename, i); // create output filename
       if ((i>istart)||(i==0)) // don't write over if restart
         write_h5part(filename, hPos, hVel, siminfo, 0);  // write h5part file

       // compute energy from gpu calculated energy vector
       if (computeE) {
         float esum = energysum(m_dPos[inew],  
                      m_dVel[inew], siminfo.softening, numBodies, p, 0.0); 
         printf("%d ensum=%.4e P=%.1f\n",i, esum,time/(2.0*M_PI)); fflush(stdout);
       }

       for (int j=0;j<siminfo.ndt;j++){  // loop timesteps for each iteration
         std::swap(iold,inew); // worked! set new to old
         // compute new positions and vels into [inew]
         integrateNbodySystem(
               m_dPos[inew], m_dVel[inew], m_dPos[iold], m_dVel[iold],
               siminfo.deltaTime, siminfo.damping, siminfo.softening,
               numBodies, p);
         time += siminfo.deltaTime;
      }
      siminfo.time = time;
   }
// store last timestep
       copyArrayFromDevice(hPos, m_dPos[inew], numBodies);
       copyArrayFromDevice(hVel, m_dVel[inew], numBodies);
       mkofilename(parmfile, filename, i); // create output filename
       write_h5part(filename, hPos, hVel, siminfo, 0);  // write h5part file
       if (computeE) {
         float esum = energysum(m_dPos[inew], 
                      m_dVel[inew], siminfo.softening, numBodies, p, 0.0); 
         printf("%d ensum=%.4e P=%.1f\n",i, esum,time/(2.0*M_PI)); fflush(stdout);
       }

   // cleanup
   deleteNBodyArrays(m_dPos);
   deleteNBodyArrays(m_dVel);
   delete [] hPos;
   delete [] hVel;

}
// create filenames for out data files
void mkofilename(char *pfile, char *ofile, int i)
{
  char junks[20];
  strcpy(ofile,pfile);
  strcpy(junks,"_");
  if (i<1000) strcat(junks,"0");
  if (i<100)  strcat(junks,"0");
  if (i<10)   strcat(junks,"0");
  strcat(ofile,junks);
  sprintf(junks,"%d",i);
  strcat(ofile,junks);
  strcat(ofile,".h5part");
//  printf("mko %s\n",ofile); fflush(stdout);
}

 
