// useful structure to store information about the simulation

#ifndef _KEPCART_
#define _KEPCART_

typedef struct {
  double x, y, z, xd, yd, zd;
} PhaseState;

typedef struct {
  double a, e, i, longnode, argperi, meananom;
} OrbitalElements;

#endif 

#define npmax 10
struct SIMinfo
{
  int numBodies;
  double deltaTime; // timestep interval
  double damping;   // damping coefficient if any
  double softening; // softening coefficient soft2
  int niter;       // total number of iterations intended
  int ndt;         // number of timesteps per interation
  double amin;     // for particle maker semi-major axis range
  double amax;     // semi-major axis range
  double gamma;    // powerlaw distribution p(a) ~ a^gamma
  double ibar;     // sqrt(<i^2>)  inclination dispersion
  double m;        // mass of non-central particles
  int    nplanets; // number of initial planets
  double mplanets[npmax];  // mass of planets (not star)
  OrbitalElements orbel[npmax];  // initial orbital elements w.r.t star
  double time;    // current simulation time since start
};

// HD5part reading and writing subroutines
// timeStep =0 is first one
// routines in hd5partstuff.cpp
  void write_h5part(char *filename, float *Pos, float *Vel, SIMinfo siminfo, int timeStep);
  void info_h5part(char *filename, SIMinfo *siminfo);
  void read_h5part(char *filename, float *Pos, float *Vel, int timeStep);

// from sread.cpp
void sread(char *parmfile,  SIMinfo *siminfo);
void correctcom(float *hPos, float *hVel, int numBodies);
// correct for center of mass drift 

extern "C"  //  these routines from bodysystemcuda.cu
{
    void checkCUDA();
    void setDeviceSoftening(float softening);
    void allocateNBodyArrays(float* vel[2], int numBodies);
    void deleteNBodyArrays(float* vel[2]);
    void allocateN(float* vec, int numBodies);
    void deleteNArray(float* vec);
    void copyArrayFromDevice(float* host, const float* device, int numBodies);
    void copyArrayToDevice(float* device, const float* host, int numBodies);
    void threadSync();
    void integrateNbodySystem(float* newPos, float* newVel,
                              float* oldPos, float* oldVel,
                              float deltaTime, float damping, float soft,
                              int numBodies, int p);
    float energysum(float* Pos,  float* Vel,
                     float soft2, int numBodies, int p, float m0);
}

